{% extends "templates/base.html" %}

{% import "macros/extra_js.html" as add_js_macros %}

{% block title %}QP: {{ array_element }} {{
subitem }}{% endblock %}

{% block content %}
<div id="page-content-wrapper">
  <div class="container-flex px-3 xyz">
    <h4>Welcome to the 'naked' {{ array_element }} page</h4>
    <p>Here there should likely be some summary results.</p>
  </div>
</div>
{% endblock %}

{% block extra_js %}
{{ add_js_macros.add_js(['d3']) }}
{% endblock %}
