{% set plot_titles = [
"pedestal_mean_charge_3D_gain_0_cameraView",
"pedestal_mean_charge_2D_gain_0",
"pedestal_mean_charge_1D_gain_0",
] %}

{% extends "templates/base.html" %}

{% import "macros/plot_grid.html" as plot_macros %}
{% import "macros/extra_js.html" as add_js_macros %}
{% import "macros/extra_css.html" as add_css_macros %}

{% block title %}
QP: {{ array_element }} {{
subitem }}
{% endblock %}

{% block extra_css %}
{{ add_css_macros.add_css(['badger','scatterplot', 'histogram1d']) }}
{% endblock %}

{% block content %}
{{ plot_macros.plot_grid(array_element, subitem, active_subitem, 2, 3, plot_titles, 3) }}
{% endblock %}

{% block extra_js %}
{{ add_js_macros.add_js(['d3','d3Hexbin','scatterplot','cameraview','histogram1d','criteriaPlot']) }}
{% endblock %}
