{% extends "templates/base.html" %}

{% block title %}QP: Page Not Found{% endblock %}

{% block content %}
<div id="page-content-wrapper">
  <div class="container-flex px-3 xyz text-center">
    <h1>404 - Page Not Found</h1>
    <p>Oops, this page doesn't exist!</p>
    <a href="/home" class="btn btn-primary">Go back Home</a>
  </div>
</div>

{% endblock %}
