.scatterplot-title {
  fill: #0066cc;
  font: bold 20px sans-serif;
  text-anchor: middle; }

.scatterplot-xlabel {
  fill: #000;
  font: 20px sans-serif;
  text-anchor: middle; }

.scatterplot-ylabel {
  fill: #000;
  font: 20px sans-serif;
  text-anchor: middle; }

.scatterplot-line {
  fill: none;
  stroke: #0066cc;
  stroke-width: 2px; }

.scatterplot-line-dashed {
  stroke-dasharray: 5, 5;
  fill: none;
  stroke: #0066cc; }

.scatterplot-line-dotted {
  stroke-dasharray: 2, 8;
  fill: none;
  stroke: #0066cc; }

.scatterplot-line-dot-dashed {
  stroke-dasharray: 6, 3, 2, 3;
  fill: none;
  stroke: #0066cc; }

.scatterplot-line-dot-dashed-dashed {
  stroke-dasharray: 6, 3, 2, 3, 6, 3;
  fill: none;
  stroke: #0066cc; }

.scatterplot-no-line {
  fill: none;
  stroke: none;
  stroke-width: 0px; }

.scatterplot-point {
  fill: #007bff;
  stroke: #fff;
  stroke-width: 1px; }

.scatterplot-xerror,
.scatterplot-yerror {
  stroke: #3399ff;
  stroke-width: 1px; }
