<div class="row">
  <div class="col-4 text-center">
    <nav aria-label="breadcrumb">
      <ol class="breadcrumb">
        <li class="breadcrumb-item">
          <a href="/{{ array_element }}">{{ array_element }}</a>
        </li>
        <li class="breadcrumb-item active" aria-current="page">
          <a href="/{{ array_element }}/{{ active_subitem }}">{{ subitem }}</a>
        </li>
      </ol>
    </nav>
  </div>

  <div class="col-8 text-center">
    <i>Selected:</i> &nbsp;&nbsp;&nbsp;&nbsp;
    Site: <strong><span id="footer-site"></span></strong> |
    Tel. type: <strong><span id="footer-tel-type"><i>None</i></span></strong> |
    Tel ID: <strong><span id="footer-tel-id" style="color: red;">Not selected</span></strong> |
    Date: <strong><span id="footer-date" style="color: red;">Not selected</span></strong> |
    OB ID: <strong><span id="footer-ob" style="color: red;">Not selected</span></strong>
  </div>
</div>
