.badger {
  margin-bottom: 10px;
  padding: 32px 15px 10px;
  border-radius: 6px;
  box-shadow: 1px 1px 5px 2px rgba(0, 0, 0, 0.1);
  border: 1px solid #99c2ff; }

.badger-min-width {
  min-width: 500px; }

[class*="badger"] {
  position: relative;
  border-radius: 6px; }

[class*="badger"]:before {
  content: attr(data-badger-left);
  position: absolute;
  top: 0;
  left: 0;
  padding: 3px 7px;
  font-size: 12px;
  font-weight: bold;
  background: #3399ff;
  color: #fff;
  border-top-left-radius: 6px;
  border-bottom-right-radius: 6px; }

[class*="badger"]:after {
  content: attr(data-badger-right);
  position: absolute;
  top: 0;
  right: 0;
  padding: 3px 7px;
  font-size: 12px;
  font-weight: bold;
  background: #3399ff;
  color: #fff;
  border-top-right-radius: 6px;
  border-bottom-left-radius: 6px; }

.badger > div {
  border-radius: 4px;
  background-color: #ddecfd; }

.badger-danger:after {
  background: #d9534f; }

.badger-warning:after {
  background: #f0ad4e; }

.badger-success:after {
  background: #5cb85c; }

.badger-info:after {
  background: #5bc0de; }

.badger-inverse:after {
  background: #222; }

.badger-primary:after {
  background: #007bff; }

.badger-null:after {
  background: #d6d8d9; }
