export const API_URL = "/api"; // backend URL
export const metadataSchemaPath = "/static/metadata_schema.yaml"; // metadata schema file path
export const criteriaSchemaPath = "/static/criteria_schema.yaml"; // autogenerated "criteria schema" file path
export const margin = { top: 30, right: 15, bottom: 60, left: 70 }; // plot margins
export const LST_CAMERA_PIXEL_COUNT = 1855;
export const lst_camera_mapping = {
  url: "/static/cameraConfig/lst_pixels_mapping_no_order.txt",
  skiplines: 6,
  format: { id: 0, x: 1, y: 2 },
};
