<!DOCTYPE html>
<html lang="en">

<head>
  <meta charset="UTF-8" />
  <meta name="viewport" content="width=device-width, initial-scale=1,  shrink-to-fit=no" />

  <title>{% block title %}QualPipe{% endblock %}</title>

  <!-- Factory CSS -->
  <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/bootstrap@4.6.2/dist/css/bootstrap.min.css"
    integrity="sha384-xOolHFLEh07PJGoPkLv1IbcEPTNtaed2xpHsD9ESMhqIYd0nLMwNLD69Npy4HI+N" crossorigin="anonymous" />
  <link rel="stylesheet" type="text/css"
    href="https://cdnjs.cloudflare.com/ajax/libs/bootstrap-datepicker/1.10.0/css/bootstrap-datepicker.min.css" />
  <link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css" />

  <!-- Factory JS (jQuery and Bootstrap Bundle includes Popper) -->
  <script src="https://code.jquery.com/jquery-3.7.1.min.js"
    integrity="sha256-/JqT3SQfawRcv/BIHPThkBvs0OEvtFFmqPF/lYI/Cxo=" crossorigin="anonymous"></script>
  <script src="https://cdn.jsdelivr.net/npm/bootstrap@4.6.2/dist/js/bootstrap.bundle.min.js"
    integrity="sha384-Fy6S3B9q64WdZWQUiU+q4/2Lc9npb8tCaSX9FK7E8HnRr0Jz8D6OP9dO5Vg3Q9ct"
    crossorigin="anonymous"></script>
  <script src="https://cdnjs.cloudflare.com/ajax/libs/bootstrap-datepicker/1.10.0/js/bootstrap-datepicker.min.js"
    integrity="sha384-hrC0I6QUg0tSdzNW47DRT7ohgBLywmjg37cLYZbdXVZLJHwJhFfqo7ouWr5yfSmv"
    crossorigin="anonymous"></script>

  <!-- Custom CSS -->
  <link href="/static/css/style.css" rel="stylesheet" />
  <link href="/static/css/sidebar.css" rel="stylesheet" />
  <link href="/static/css/footer.css" rel="stylesheet" />
  <!-- Custom JS -->
  <script src="/static/js/base.js" type="module"></script>
  <script src="/static/js/sidebarAndFooter.js" type="module"></script>
  <!-- Schema validation related -->
  <script src="/static/lib/js/ajv.min.js"></script>
  <script src="/static/lib/js/js-yaml.min.js"></script>

  {% block extra_css %}{% endblock %}
</head>

<body class="d-flex flex-column h-100">
  <header>
    {% include 'templates/navigationBar.html' %}
  </header>

  <main role="main" class="flex-shrink-0">
    <div id="wrapper" class="collapsed">
      {% include 'templates/sideBar.html' %}
      {% block content %}{% endblock %}
    </div>
  </main>
  <footer class="footer fixed-bottom text-center">
    {% include 'templates/footer.html' %}
  </footer>
</body>

{% block extra_js %}{% endblock %}

</html>
