<div id="sidebar-wrapper">
  <ul class="sidebar-nav nav-pills nav-stacked mt-1" id="menu">
    <li>
      <a href="#"><span class="fa-stack fa-lg"><i class="fa fa-globe fa-stack-1x"></i></span>
        <label for="which-Site"></label>
        <select class="form-control no-focus" id="which-Site" name="which-Site-list" style="
                text-align-last: center;
                display: inline;
                font-weight: bold;
              ">
          <option value="North" {% if array_element=='SSTs' %}disabled{% endif %}>North</option>
          <option value="South" {% if array_element=='SSTs' %}selected{% endif %}>South</option>
        </select></a>
    </li>

    <li>
      <a href="#"><span class="fa-stack fa-lg">
          <i class="fa fa-calendar fa-stack-1x"></i>
        </span><label for="which-date"></label>
        <input id="date-picker" class="form-control no-focus" placeholder="Choose a date"
          style="text-align-last: center; display: inline; font-weight: bold;" />
      </a>
    </li>

    {% if array_element=='LSTs' or array_element=='MSTs' or array_element=='SSTs' %}
    <li>
      <a href="#">
        <span class="fa-stack fa-lg">
          <strong>{# <i class="fa fa-list-ol fa-stack-1x"></i> #} OB
          </strong></span>
        <label for="which-OB"></label>
        <select class="form-control no-focus" id="which-OB" name="which-OB-list" style="
                text-align-last: center;
                display: inline;
                font-weight: bold;
              ">
          <option value="0" disabled>choose date first</option>
        </select></a>
    </li>
    {% endif %}

    {% if array_element=='LSTs' or array_element=='MSTs' or array_element=='SSTs' %}
    <li>
      <a href="#">
        <span class="fa-stack fa-lg">
          <i class="fa fa-hashtag fa-stack-1x"></i>
        </span>
        <label for="which-Tel-ID"></label>
        <select class="form-control no-focus" id="which-Tel-ID" name="which-Tel-ID-list" style="
                text-align-last: center;
                display: inline;
                font-weight: bold;
              ">
          <option value="0" disabled>select a Tel ID</option>
        </select></a>
    </li>
    {% endif %}

    <li>&nbsp;</li>

    {% if array_element=='LSTs' or array_element=='MSTs' or array_element=='SSTs' %}
    <li>
      <a href="#">
        <span class="fa-stack fa-lg">
          <i class="fa fa-line-chart fa-stack-1x"></i>
        </span>
        <button type="button" id="makePlot" class="btn btn-primary">
          Make Plot
        </button>
      </a>
    </li>
    {% endif %}

    <li>&nbsp;</li>

    <li>
      <span id="missingInfo" style="display: block; text-align: center; color: white;"></span>
    </li>
  </ul>
</div>
