.cameraview-title {
  fill: #0066cc;
  font: bold 34px sans-serif;
  text-anchor: middle; }

.special_hexagon {
  stroke: #ff0000 !important; }

.palette-selector-container {
  position: absolute;
  bottom: 5px;
  left: 5px;
  z-index: 1000;
  background: transparent;
  padding: 5px 5px; }

.label-palette {
  font-size: 14px;
  margin-right: 5px; }

.palette-selector {
  font-size: 14px;
  padding: 2px 5px;
  cursor: pointer;
  border: 1px solid #ccc;
  border-radius: 3px; }
