.histogram1d-title {
  fill: #0066cc;
  font: bold 20px sans-serif;
  text-anchor: middle;
}

.histogram1d-xlabel {
  fill: #000;
  font: 20px sans-serif;
  text-anchor: middle;
}

.histogram1d-ylabel {
  fill: #000;
  font: 20px sans-serif;
  text-anchor: middle;
}

.histogram1d-line {
  fill: none;
  stroke: #0066cc;
  stroke-width: 2px;
}

.histogram1d-bar {
  fill: #007bff;
  stroke: #fff;
  stroke-width: 1px;
}

.histogram1d-yerror {
  stroke: #3399ff;
  stroke-width: 1px;
}
